#!/bin/sh
#
# Teng -- a general purpose templating engine.
# Copyright (C) 2004  Seznam.cz, a.s.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Seznam.cz, a.s.
# Naskove 1, Praha 5, 15000, Czech Republic
# http://www.seznam.cz, mailto:teng@firma.seznam.cz
#
#
# $Id: test.sh,v 1.2 2006/07/11 14:46:20 sten__ Exp $
#
# DESCRIPTION
# Script for executing tests.
#
# AUTHORS
# Jan Adamek <jan.adamek@firma.seznam.cz>
#
# HISTORY
# 2006-07-11  (sten__)
#             Created.
#
# USAGE
# ./test.sh (all|error|expression|fragment|function)
#

# Minimal required version
REQUIRED=1.0.13

run_test() {
    if ! [ -f "$1.teng" ]; then
        echo -n " unable to locate template,"
        return 1
    fi
    if ! [ -f "$1.exp" ]; then
        echo -n " unable to locate expected output,"
        return 1
    fi
    if ! [ -f "$1.xml" ]; then
        echo -n " unable to locate data source,"
        return 1
    fi
    DICT=
    if [ -f "$1.cs.dict" ]; then
        DICT="-dict $1.dict"
    fi
    CONF=
    if [ -f "$1.param" ]; then
        CONF="-conf $1.param"
    fi
    $EXEC -exp $1.exp -templ $1.teng -xml $1.xml -lang cs $DICT $CONF -out $1.out
    return $?
}

run_tests() {
    ok=0
    for test in $1/*.xml; do
        if [ "$test" = "$1/*.xml" ]; then
            echo "${TEST_PREFIX}    No tests found"
            return 0
        fi
        i=`echo -n $test | sed 's/\.xml$//'`
        num=`echo -n $i | sed 's/^[^1-9]*//'`
        echo -n "${TEST_PREFIX}    Running test #$num..."
        run_test $i
        e=$?
        if [ "$e" -eq "0" ]; then
            echo -n " successful"
        elif [ "$e" -gt "128" ]; then
            echo -n " successful with all expected errors"
        else
            ok=1
            echo -n " failed"
        fi
        echo " (see report file $i.out)"
    done
    return $ok
}

if ! [ -d "error" -a -d "expression" -a -d "fragment" -a -d "function" ]; then
    echo "This file must be run in the directory with tests"
fi

if [ -x "../python/generate.py" ] && ../python/generate.py -help -quiet; then
    EXEC="../python/generate.py"
else
    echo "Cannot execute tests before Teng is installed"
    exit 1
fi

version="0.0.0"
for f in "/usr/local/lib/libteng.so" "/usr/lib/libteng.so"; do
    if [ -f $f ]; then
        version=$(ls -l $f | sed 's/^.*\.so\.\([^ ]*\)$/\1/')
        break
    fi
done

if [ "$TEST_PREFIX" = "" ]; then
    if [ "$(echo -n "$version" | cut -d "." -f 1 )" -le "$(echo -n "$REQUIRED" | cut -d "." -f 1 )" ]; then
        if [ "$(echo -n "$version" | cut -d "." -f 2 )" -le "$(echo -n "$REQUIRED" | cut -d "." -f 2 )" ]; then
            if [ "$(echo -n "$version" | cut -d "." -f 3 )" -lt "$(echo -n "$REQUIRED" | cut -d "." -f 3 )" ]; then
                if [ "$version" = "0.0.0" ]; then
                    echo "WARNING: This test suite requires Teng library version $REQUIRED or newer. You're running unknown version."
                else
                    echo "ERROR: This test suite requires Teng library version $REQUIRED or newer. You're running version $version."
                    exit 1
                fi
                echo
            fi
        fi
    fi
fi

case "$1" in
    all)
        echo "Executing tests..."
        error=0
        for i in "fragment" "expression" "function" "error"; do
            if ! TEST_PREFIX="    " $0 $i; then
                error=1
            fi
        done
        if [ "$error" -eq "0" ]; then
            echo "All tests were successful"
        else
            echo "Some tests failed"
        fi
        exit $error
        ;;
    error|expression|fragment|function)
        echo "${TEST_PREFIX}Executing $1 tests..."
        if run_tests $1; then
            echo "${TEST_PREFIX}All $1 tests were successful"
            exit 0
        else
            echo "${TEST_PREFIX}Some $1 tests failed"
            exit 1
        fi
        ;;
    *)
        echo "Usage: $0 (all|error|expression|fragment|function)" >&2
        exit 1
        ;;
esac
